#include <HX711_ADC.h>
#include "Arduino.h"
#include "SoftwareSerial.h"
#include "DFRobotDFPlayerMini.h"

// 震动马达模块针脚-数据参数 定义
#define motorPin      11              // 震动马达的输出针脚
#define motorStrength 200             // 震动马达的强度值
unsigned long motorrunTime = 0;       // 存储记录时间
bool motorFlag = false;               // 震动马达 true为启用状态  false为关闭
#define motorTimeMax (60 * 1000UL)    // 拿走手机后马达震动时间60秒
#define motorTimeMin (6 * 1000UL)     // 达到时间点后震动时间6秒(三组)
bool motorrunState = true;            // 启用点缀（只执行一次）在系统启动后，手机被拿走启用
bool motorcloseState = true;          // 启用点缀（只执行一次）在系统启动后，完成学习任务后启用
uint8_t motorSystemState = 0x00;           // 马达震动状态
unsigned long motorSystemTime;             // 存储马达初始化时间

// 触摸模块针脚-数据参数 定义
#define touchPin      5               // 触摸模块的输入针脚
#define touchInterval 200             // 设置多少毫秒检测一次触摸模块状态变化
bool touchState = false;              // 检测到触摸后，更改状态为启用，控制系统开启
unsigned long touchTime = 0;          // 触摸检测间隔时间
bool touchFlag = true;                // 启用检测后，将关闭触摸模块；true 为检测开  false 为检测关

// 称重模块针脚-数据参数 定义
#define HX711Interval 100             // 称重模块检测间隔时间
#define weightMin     150             // 称重重量间隔小值大于150则为有手机
#define weightMax     500             // 称重重量间隔大值并且小于500为正常状态
#define hx711dout     3               // mcu > HX711 dout pin
#define hx711sck      2               // mcu > HX711 sck pin

// 宏定义针脚端口
HX711_ADC LoadCell(hx711dout, hx711sck);   // HX711构造函数：
unsigned long HX711Time = 0;               // 运行间隔时间

// 时间定义参数
//#define runTimeMax (60*60*1000UL)        // 系统整体运行时长(60*60*1000UL)
unsigned long runTimeMax (60*60*1000UL); // 系统整体运行时长(60*60*1000UL)
unsigned long runTime = 0;                 // 存储系统记录时间
bool runTimeState = true;                  // 系统启用记录时间 点段记录
unsigned long lazyTime = 0;                // 存储偷懒时间

// MP3模块定义
#define myDFPlayerTX   8
#define myDFPlayerRX   9
SoftwareSerial mySoftwareSerial(myDFPlayerTX, myDFPlayerRX);          // TX, RX
DFRobotDFPlayerMini myDFPlayer;

void setup() {
  Serial.begin(9600);                      // 串口波特率9600
  mySoftwareSerial.begin(9600);            // MP3模块通讯
  pinMode(motorPin, OUTPUT);               // 震动马达针脚输出
  digitalWrite(motorPin, LOW);             // 初始化震动马达关闭

  pinMode(touchPin, INPUT);                // 触摸模块输入

  if (!myDFPlayer.begin(mySoftwareSerial)) {             // 使用softwareSerial与mp3通信。
    Serial.println(F("无法开始："));
    Serial.println(F("1.请重新检查连接！"));
    Serial.println(F("2.请插入SD卡！"));
    while (true) {
      delay(0);
    }
  }

  myDFPlayer.volume(30);                                // 初始化音量大小

  Serial.println("系统启动中...");
  LoadCell.setCalFactor(-449.61);                       // 设置校准系数（浮点）
  LoadCell.begin();
  Serial.println("初始化称重模块");
  LoadCell.tareNoDelay();
  Serial.println("初始化完成, 系统启动完成.");


  Serial.print("motorTimeMax"); Serial.println(motorTimeMax);
  Serial.print("motorTimeMin"); Serial.println(motorTimeMin);
  Serial.print("runTimeMax"); Serial.println(runTimeMax);
}

void loop() {
  // 触摸模块检测，如若检测为高电平，则为已触摸，则启用系统！
  getTouch();

  // HXT700 模块重量检测
  static bool newDataReady = 0;                // 定义是否有数据变化的布尔值
  if (LoadCell.update()) {                     // 检查新数据/启动下一个转换：
    newDataReady = true;
  }
  float weight;                                // 存储重量
  if (newDataReady) {
    if (millis() - HX711Time > HX711Interval) {
      HX711Time = millis();
      weight = LoadCell.getData();
      if ( weight < 0.5 ) {
        weight = 0;
      }
      //      Serial.print("重量："); Serial.print(weight); Serial.println("g");
    }
  }


  // 逻辑判断  1. 当客户放入手机，重量大于150g 小于500g  并且触摸检测开始时，系统计时开启
  if (weight > weightMin && weight < weightMax && touchState) {
    // 调用模块记录当前时间 记录如若为1小时后，调整触摸开启状态，触摸系统关闭
    if (runTimeState) {                                          // 状态间隔记录时间
      runTime = millis();                                        // 记录启动时间点
      myDFPlayer.playFolder(1, 1);                               // 播放系统启用音乐
      runTimeState = false;                                      // 关闭间隔记录，等待下一次存储值
    }

    // 检测是否存在懒惰值，如若有在整体系统时间上相加，如果没有则不进入；（检测如若有懒惰值，则相加后，清除懒惰值）
    if (lazyTime > 0) {
      runTimeMax += lazyTime;
      lazyTime = 0;
      Serial.print("系统整体运行时间："); Serial.println(runTimeMax);
    }

    if (millis() - runTime <= runTimeMax) {
      // 初始化关闭后的状态值
      motorFlag = false;                       // 客户在多次拿走手机又放入后，关闭启用提醒
      getMotor(0);                             // 调用马达关闭命令
      motorrunState = true;                    // 初始化调用客户在关闭后重新放入手机开始计时的动态更新
      motorcloseState = true;                  // 系统启用后，初始化状态数值(需要检测时间到期后开启状态)

      Serial.println("我在学习中");
    }
    else {
      if (motorcloseState) {
        motorcloseState = false;               // 更改状态，时间进来后关闭
        motorrunTime = millis();               // 获取当前系统时间,用于马达计算测试时间
        motorFlag = true;
      }
      Serial.println("学习任务已经完成");
      touchState = false;                      // 关闭触摸模块，关闭系统
      runTimeState = true;                     // 系统闭环完成后，初始化系统时间启动记录值
      touchFlag = true;                        // 系统闭环完成后，初始化触摸模块检测状态，用于下次启用
      runTimeMax = 3600000;                    // 初始化数值变量
      myDFPlayer.playFolder(3, 1);             // 播放学习任务完成音乐
    }
  }

  // 逻辑判断  2. 当客户放入手机系统启动计时后，发现在期间内手机被老六给拿走了，就开始震动模块的功能启用，并且记录时间，重新加入到系统启用期间中
  else if (weight < weightMin && touchState) {
    motorFlag = true;                          // 开启震动系统

    // 启用时间记录方法，并且需要将此期间值加入到整体系统运行时间中
    if (motorrunState) {
      motorrunState = false;                   // 更改状态，进来后关闭
      motorrunTime = millis();                 // 获取当前系统时间,用于马达计算测试时间
      myDFPlayer.playFolder(2, 1);             // 播放手机被拿走提示音乐
    }
    lazyTime = millis() - motorrunTime;        // 计算时间差
    Serial.println("你个老六放下手机，赶紧学习！");
    Serial.print("系统增加延长时间为："); Serial.println(lazyTime);

  }

  // 震动马达测试模块
  /*
    motorFlag = true;                        // 开启震动系统
    motorrunTime = millis();                 // 获取当前系统时间
  */
  if (motorFlag) {
    if (weight < weightMin) {
      getMotor(motorTimeMax);                 // 手机在开启系统后，被老六给拿走了，调用启用方法60秒震动
    }
    else {
      getFrequencyMotor(motorTimeMin);        // 你已经完成了学习任务，该休息了(震动一秒 休息一秒)
    }
  }


  // loop模块回括号
}

// 震动马达调用时间控制模块
void getMotor(long motorTime) {
  if ( motorTime == 0 ) {                         // 判断方法带入值为0，则关闭震动马达模块
    digitalWrite(motorPin, LOW);                  // 关闭震动马达模块
  }
  else {
    //      Serial.print("按T后所记录的时间："); Serial.println(motorrunTime);
    if (millis() - motorrunTime <= motorTime) {      // 计时器，让震动马达在此时间段内工作，控制使用
      //      motorrunTime = millis();                        // 获取系统方法时间并做记录
      //      Serial.println(motorrunTime);
      Serial.println("震动开始");
      analogWrite(motorPin, motorStrength);         // 震动马达PWM输出
    } else {
      motorFlag = false;
      digitalWrite(motorPin, LOW);                  // 关闭震动马达模块
    }
  }
}


// 震动马达调用时间控制模块 结束后的频率震动
void getFrequencyMotor(long motorTime) {
  if ( motorTime == 0 ) {                         // 判断方法带入值为0，则关闭震动马达模块
    digitalWrite(motorPin, LOW);                  // 关闭震动马达模块
  }
  else {
    if (millis() - motorrunTime <= motorTime) {      // 计时器，让震动马达在此时间段内工作，控制使用
      Serial.println("震动开始");
      switch (motorSystemState) {
        case 0x00: {
            if (millis() - motorSystemTime >= 1000) {
              motorSystemTime = millis();
              analogWrite(motorPin, motorStrength);         // 震动马达PWM输出
              Serial.println("震动一秒");
              motorSystemState ++ ;
            }
          } break;
        case 0x01: {
            if (millis() - motorSystemTime >= 1000) {
              motorSystemTime = millis();
              analogWrite(motorPin, 0);         // 震动马达PWM输出
              Serial.println("休息一秒");
              motorSystemState = 0x00;
            }
          } break;
        default: break;
      }
    } else {
      motorSystemTime = 0;
      motorSystemState = 0x00;
      motorFlag = false;
      digitalWrite(motorPin, LOW);                  // 关闭震动马达模块
    }
  }
}

// 触摸模块检测，更改状态值
void getTouch() {
  if (touchFlag) {
    if (millis() - touchTime > touchInterval) {
      touchTime = millis();
      if (digitalRead(touchPin) == HIGH) {          // 检测触摸模块输出是否为高电平
        touchState = true;                          // 系统启用
        Serial.println("系统开启，强制学习");          // 如果为高电平，则表示开启
        touchFlag = false;                          // 系统检测启动后，关闭触摸模块
      }
    }
  }
}
